#!/bin/bash
# IBM_PROLOG_BEGIN_TAG
#
# IBM CONFIDENTIAL
#
# OBJECT CODE ONLY SOURCE MATERIALS
#
# Program 5724-K69
#
# COPYRIGHT International Business Machines Corp. 2004,2008
# All Rights Reserved
#
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.
#
# IBM_PROLOG_END_TAG


display=""
wall=""
transport=""
mpicom=""
mpibin=""
sync=0
pointer=0
selector=0
clientrender=0
swap=0
localaddr=""
isdmx=0

while test X$1 != X ; do
    case $1 in
        -display)
            shift
            display=$1 ;;
        -wall)
            shift
            wall=$1 ;;
        -transport)
            shift
            transport=$1 ;;
        -mpicomm)
            shift
            mpicomm=$1 ;;
        -mpibin)
            shift
            mpibin=$1 ;;
        -sync)
            shift
            sync=$1 ;;
        -pointer)
            shift
            pointer=$1 ;;
        -selector)
            shift
            selector=$1 ;;
        -clientrender)
            shift
            clientrender=$1 ;;
        -swap)
            shift
            swap=$1 ;;
        -localaddr)
            shift
            localaddr=$1 ;;
        -dmx)
            isdmx=1 ;;
        -*)
                echo 'Error: The SVN option $1 is not recognized.\n'
                exit 20;;
    esac
    shift
done

SVN_CONNECT_COMMAND=`which ssh`
SVN_COPY_COMMAND=`which scp`
if [ -z "$SVN_REMOTE_USER" ] ; then
    SVN_REMOTE_USER=`id -nu`
fi

errors=0

# check that local display is accessible

if test "$display" != "" ; then
    /usr/X11R6/bin/xdpyinfo -display $display >/dev/null 2>/dev/null
    if test $? -ne 0 ; then
        echo "ERROR: cannot access display \"$display\"."
        errors=`expr $errors + 1`
    fi
else
    echo "ERROR: display is not set."
    errors=`expr $errors + 1`
fi

# check if wall.cfg file is specified and exists (if not dmx)
# create /tmp/$$.screens containing machine name list

if test $isdmx -eq 0 ; then
    if test "$wall" = ""; then
                echo "ERROR: you must define a wall.cfg file"
        errors=`expr $errors + 1`
        touch /tmp/$$.screens
    else
        cat $wall | grep -v '#' | grep -v '^$' | awk '{ print $1 }'  > /tmp/$$.screens
    fi
else
    if test "$wall" != ""; then
        echo "WARNING: you specified a wall.cfg file but it will not be used since your display is DMX"
        wall=""
    fi
    /opt/IBM/dcv/svn/bin/svn_dmx_server_info $display > /tmp/$$.screens
    if test $? -ne 1 ; then
        echo "ERROR: cannot access DMX display $display."
        errors=`expr $errors + 1`
    fi
    # not check that $display contains an hostname
	thost=`echo $display | gawk --field-separator=: "{ print \\\$1 }"`

	if test "$thost" != ""; then
        if test "$thost" == "localhost"; then
	        echo "ERROR: your DISPLAY variable ($display) should contain the hostname of the DMX display."
	        errors=`expr $errors + 1`
        fi
	else
        echo "ERROR: your DISPLAY variable ($display) should contain the hostname of the DMX display."
        errors=`expr $errors + 1`
	fi
fi

# check that all remote display can be reached

for i in `cat /tmp/$$.screens`
  do
  /usr/X11R6/bin/xdpyinfo -display $i >/dev/null 2>/dev/null
  if test $? -ne 0 ; then
      echo "ERROR: cannot access display \"$i\", check your wall.cfg file."
      errors=`expr $errors + 1`
  fi
done

if test "$wall" != "" ; then
    if test ! -f "$wall" ; then
        echo "ERROR: cannot find file \"$wall\"."
        errors=`expr $errors + 1`
    fi
fi

# check that sync on swap is between 0 and 3

if test $sync -gt 3 ; then
    echo "ERROR: -sync value ($sync) must be between 0 and 3."
    errors=`expr $errors + 1`
else
    if test $sync -lt 0 ; then
        echo "ERROR: -sync value ($sync) must be between 0 and 3."
        errors=`expr $errors + 1`
    fi
fi

# check that transport is MPI, SOCKET or an existing file

case "$transport" in
    MPI)     ;;
    OPENMPI) ;;
    SOCKET)  ;;
    *)
        if test "$transport" = "" ; then
            echo "ERROR: empty transport is invalid."
            errors=`expr $errors + 1`
        else
            if test ! -e $transport ; then
                echo "ERROR: the transport \"$transport\" is invalid."
                errors=`expr $errors + 1`
            fi
        fi
esac

if test "$transport" = "MPI" ; then
    # if MPI check that mpicomm is valid

    case "$mpicomm" in
        mpicomm.ib.rsh) ;;
        mpicomm.ib.rsh) ;;
        mpicomm.p4) ;;
        mpicomm.scali) echo "WARNING: Scali MPI is not officially supported." ;;
        *)
            if test "$mpicomm" = "" ; then
                echo "ERROR: must specify an mpicomm script for use MPI."
                errors=`expr $errors + 1`
            else
                if test ! -f "$mpicomm" ; then
                    if test ! -f "/opt/IBM/dcv/svn/bin/$mpicomm" ; then
                        echo "ERROR: mpicomm script \"$mpicomm\" does not exist."
                        errors=`expr $errors + 1`
                    else
                        echo "WARNING: using unsupported mpicomm script \"/opt/IBM/dcv/svn/bin/$mpicomm\"."
                    fi
                else
                    echo "WARNING: using unsupported mpicomm script \"$mpicomm\"."
                fi
    	    fi
    esac

    # if MPI check if libmpich.so is in the path for 32 bit

        ldd /opt/IBM/dcv/svn/lib/libsvntrn_cli_mpi.so.1 | grep "not found" >& /dev/null
    if test $? -eq 0 ; then
        echo "ERROR: cannot load 32 bit MPI transport, probably libmpich.so is not in library path"
		errors=`expr $errors + 1`
    fi

    # if MPI check if libmpich.so is in the path for 64 bit

    if test -e /opt/IBM/dcv/svn/lib64/libsvntrn_cli_mpi.so.1 ; then
        ldd /opt/IBM/dcv/svn/lib64/libsvntrn_cli_mpi.so.1 | grep "not found" >& /dev/null
        if test $? -eq 0 ; then
			echo "ERROR: cannot load 64 bit MPI transport, probably libmpich.so is not in library path"
			errors=`expr $errors + 1`
        fi
  fi

    # if MPI check that the mpi command is in the path or in mpibin

    case "$mpicomm" in
        mpicomm.ib.rsh)
            if test "$mpibin" = "" ; then
                which mpirun_rsh >& /dev/null
                if test $? -ne 0 ; then
                    echo "ERROR: -mpibin not used and mpirun_rhs command not in system PATH."
                    errors=`expr $errors + 1`
                fi
            else
                if test ! -f "$mpibin/mpirun_rsh" ; then
                    echo "ERROR: mpirun_rsh command not present in mpibin directory \"$mpibin\"."
                    errors=`expr $errors + 1`
                fi
            fi
            ;;
        mpicomm.ib.ssh)
            if test "$mpibin" = "" ; then
                which mpirun_ssh >& /dev/null
                if test $? -ne 0 ; then
                    echo "ERROR: -mpibin not used and mpirun_ssh command not in system PATH."
                    errors=`expr $errors + 1`
                fi
            else
                if test ! -f "$mpibin/mpirun_ssh" ; then
                    echo "ERROR: mpirun_ssh command not present in mpibin directory \"$mpibin\"."
                    errors=`expr $errors + 1`
                fi
            fi
            ;;
        mpicomm.p4)
            if test "$mpibin" = "" ; then
                which mpirun >& /dev/null
                if test $? -ne 0 ; then
                    echo "ERROR: -mpibin not used and mpirun command not in system PATH."
                    errors=`expr $errors + 1`
                fi
            else
                if test ! -f "$mpibin/mpirun" ; then
                    echo "ERROR: mpirun command not present in mpibin directory \"$mpibin\"."
                    errors=`expr $errors + 1`
                fi
            fi
            ;;
        mpicomm.scali)
            if test "$mpibin" = "" ; then
                which mpimon >& /dev/null
                if test $? -ne 0 ; then
                    echo "ERROR: -mpibin not used and mpimon command not in system PATH."
                    errors=`expr $errors + 1`
                fi
            else
                if test ! -f "$mpibin/mpimon" ; then
                    echo "ERROR: mpimon command not present in mpibin directory \"$mpibin\"."
                    errors=`expr $errors + 1`
                fi
            fi
            ;;
    esac

    if test -e "$SVN_CONNECT_COMMAND" ; then
        for j in `cat /tmp/$$.screens`
          do
          i=`echo $j | gawk --field-separator=: "{ print \\\$1 }"`

          $SVN_CONNECT_COMMAND "${SVN_REMOTE_USER}@${i}" "exit 3" >/dev/null
          if test $? -eq 3 ; then
              
          # if MPI check if libmpich.so is in the path for 32 bit on remote machines
              
              $SVN_CONNECT_COMMAND "${SVN_REMOTE_USER}@${i}" "ldd /opt/IBM/dcv/svn/lib/libsvntrn_srv_mpi.so.1" | grep "not found" >& /dev/null
              if test $? -eq 0 ; then
                  echo "ERROR: cannot load 32 bit MPI transport on machine $i, probably libmpich.so is not in library path"
              errors=`expr $errors + 1`
              fi
              
          # if MPI check if libmpich.so is in the path for 64 bit on remote machines
              
              if test -e /opt/IBM/dcv/svn/lib64/libsvntrn_cli_mpi.so.1 ; then
                  $SVN_CONNECT_COMMAND "${SVN_REMOTE_USER}@${i}" "ldd /opt/IBM/dcv/svn/lib64/libsvntrn_srv_mpi.so.1" | grep "not found" >& /dev/null
                  if test $? -eq 0 ; then
                      echo "ERROR: cannot load 64 bit MPI transport on machine $i, probably libmpich.so is not in library path"
                      errors=`expr $errors + 1`
                  fi
              fi
          else
              echo "WARNING: cannot verify MPI libraries on $i: ssh connection failed."
          fi
        done
    else
        echo "WARNING: cannot verify MPI libraries on $i: ssh command not available."
    fi
fi

if test "$transport" = "OPENMPI" ; then
    # if MPI check that mpicomm is valid

    case "$mpicomm" in
        mpicomm.openmpi) ;;
        *)
            if test "$mpicomm" = "" ; then
                echo "ERROR: must specify an mpicomm script for use MPI."
                errors=`expr $errors + 1`
            else
                if test ! -f "$mpicomm" ; then
                    if test ! -f "/opt/IBM/dcv/svn/bin/$mpicomm" ; then
                        echo "ERROR: mpicomm script \"$mpicomm\" does not exist."
                        errors=`expr $errors + 1`
                    else
                        echo "WARNING: using unsupported mpicomm script \"/opt/IBM/dcv/svn/bin/$mpicomm\"."
                    fi
                    else
                        echo "WARNING: using unsupported mpicomm script \"$mpicomm\"."
                    fi
                fi
            esac

    # if MPI check if libmpi.so is in the path for 32 bit

    ldd /opt/IBM/dcv/svn/lib/libsvntrn_cli_openmpi.so.1 | grep "not found" >& /dev/null
    if test $? -eq 0 ; then
          echo "ERROR: cannot load 32 bit MPI transport, probably libmpi.so is not in library path"
          errors=`expr $errors + 1`
    fi

    # if MPI check if libmpi.so is in the path for 64 bit

    if test -e /opt/IBM/dcv/svn/lib64/libsvntrn_cli_openmpi.so.1 ; then
        ldd /opt/IBM/dcv/svn/lib64/libsvntrn_cli_openmpi.so.1 | grep "not found" >& /dev/null
        if test $? -eq 0 ; then
          echo "ERROR: cannot load 64 bit MPI transport, probably libmpi.so is not in library path"
          errors=`expr $errors + 1`
        fi
  fi

    # if MPI check that the mpi command is in the path or in mpibin

    case "$mpicomm" in
        mpicomm.openmpi)
            if test "$mpibin" = "" ; then
                which mpirun >& /dev/null
                if test $? -ne 0 ; then
                    echo "ERROR: -mpibin not used and mpirun command not in system PATH."
                    errors=`expr $errors + 1`
                fi
            else
                if test ! -f "$mpibin/mpirun" ; then
                    echo "ERROR: mpirun command not present in mpibin directory \"$mpibin\"."
                    errors=`expr $errors + 1`
                fi
            fi
            ;;
    esac

    if test -e "$SVN_CONNECT_COMMAND" ; then
        for j in `cat /tmp/$$.screens`
          do
          i=`echo $j | gawk --field-separator=: "{ print \\\$1 }"`

          $SVN_CONNECT_COMMAND "${SVN_REMOTE_USER}@${i}" "exit 3" >/dev/null
          if test $? -eq 3 ; then
              
          # if MPI check if libmpi.so is in the path for 32 bit on remote machines
              
              $SVN_CONNECT_COMMAND "${SVN_REMOTE_USER}@${i}" "ldd /opt/IBM/dcv/svn/lib/libsvntrn_srv_openmpi.so.1" | grep "not found" >& /dev/null
              if test $? -eq 0 ; then
                  echo "ERROR: cannot load 32 bit MPI transport on machine $i, probably libmpi.so is not in library path"
              errors=`expr $errors + 1`
              fi
              
          # if MPI check if libmpi.so is in the path for 64 bit on remote machines
              
              if test -e /opt/IBM/dcv/svn/lib64/libsvntrn_cli_openmpi.so.1 ; then
                  $SVN_CONNECT_COMMAND "${SVN_REMOTE_USER}@${i}" "ldd /opt/IBM/dcv/svn/lib64/libsvntrn_srv_openmpi.so.1" | grep "not found" >& /dev/null
                  if test $? -eq 0 ; then
                      echo "ERROR: cannot load 64 bit MPI transport on machine $i, probably libmpi.so is not in library path"
                      errors=`expr $errors + 1`
                  fi
              fi
          else
              echo "WARNING: cannot verify MPI libraries on $i: ssh connection failed."
          fi
        done
    else
        echo "WARNING: cannot verify MPI libraries on $i: ssh command not available."
    fi

fi

if test "$transport" = "SOCKET" ; then
    
    if test ! -e "$SVN_CONNECT_COMMAND" ; then
        echo "ERROR: SOCKET transport needs ssh command."
        errors=`expr $errors + 1`
    fi

    if test ! -e "$SVN_COPY_COMMAND" ; then
        echo "ERROR: socket transport needs scp command."
        errors=`expr $errors + 1`
    fi

    # if socket check that we can ssh into remote machines

    for j in `cat /tmp/$$.screens`
      do
      i=`echo $j | gawk --field-separator=: "{ print \\\$1 }"`

      $SVN_CONNECT_COMMAND "${SVN_REMOTE_USER}@${i}" "exit 3" >/dev/null
      if test $? -ne 3 ; then
          echo "ERROR: cannot access machine \"$i\", check your ssh config."
          errors=`expr $errors + 1`
      fi
    done

    # check that wall and localaddr are on same interface type 

    local=`echo $display | gawk --field-separator=: "{ print \\\$1 }"`
    if test "$local" = "" ; then
        local=`hostname -s`
    fi

    # HERE TODO
fi

# cleanup

rm /tmp/$$.screens

# print summary

if test $errors -gt 1 ; then
    echo
    echo "svn_check found $errors errors. SVN will probably not work with this configuration."
    exit 20
fi

if test $errors -eq 1 ; then
    echo
    echo "svn_check found $errors error. SVN will probably not work with this configuration."
    exit 20
fi

exit 0





