/* IBM_PROLOG_BEGIN_TAG                                                   */
/* This is an automatically generated prolog.                             */
/*                                                                        */
/*                                                                        */
/*                                                                        */
/* IBM CONFIDENTIAL                                                       */
/*                                                                        */
/* OBJECT CODE ONLY SOURCE MATERIALS                                      */
/*                                                                        */
/* COPYRIGHT International Business Machines Corp. 2005, 2007             */
/* All Rights Reserved                                                    */
/*                                                                        */
/* The source code for this program is not published or otherwise         */
/* divested of its trade secrets, irrespective of what has been           */
/* deposited with the U.S. Copyright Office.                              */
/*                                                                        */
/* IBM_PROLOG_END_TAG                                                     */

// ABOUT:
// This overload demonstrates a server only Overload to alter the size
// of rendered points on the display wall

#include <stdio.h>
#include <stdlib.h>
// wire.h contains the SVN OpenGL enumerants (e.g. GLCOLOR3F etc.) and must be included
#include "wire.h"

// Overload glBegin(GL_POINTS) and glEnd()
static void (*s_glBegin)(GLenum);
static void   o_glBegin(GLenum);
static void (*s_glEnd)(void);
static void   o_glEnd(void);
static void (*s_glPointSize)(GLfloat);
static void (*s_glGetFloatv)(GLenum, GLfloat*);

float pointSize     = 2.0; // Default. value of SVN_BIG_POINTS_SIZE is read also
float oldPointSize  = 0.0;



void Overload(glOp *glOperationsTable, glOp *glSystemOperationsTable)
{
    // Acquire a function pointer to access the DVGetDisplayName() SVN function
    char*(*_getDisplayName)() = 
        (char*(*)()) glOperationsTable[DVGETDISPLAYNAME];

    char* myDisplayName = _getDisplayName();

    // For Server Nodes myDisplayName will be the system's hostname as specified 
    // in the wall configuration file.
    // For the Client Application Node, myDisplayName will be "client".
    // If an error occurred myDisplayName will be NULL.
    if (myDisplayName==NULL)
    {
        fprintf(stderr,"Failed to obtain SVN system info;\n");
        return;
    }
    else if ( !strcmp(myDisplayName,"client") )
    {
        // Do not apply overload to the client side
        return;
    }

    fprintf(stderr, "SVN server rendering on display %s using \"Big Points\" Overload... \n", myDisplayName);

    char *c;
    c = getenv("SVN_BIG_POINTS_SIZE");
    if(c){
        pointSize = atof(c);
    }
    // Acquire a pointer to the 'true' OpenGL function for future use.
    // glOperationsTable are 'true' functions with SVN internal alterations
    // glSystemOperationsTable are the functions direct from the OpenGL driver
    s_glBegin = (void(*)(GLenum)) glOperationsTable [GLBEGIN];
    s_glEnd   = (void(*)(void))   glOperationsTable [GLEND];
    s_glPointSize = (void(*)(GLfloat)) glSystemOperationsTable[GLPOINTSIZE];
    s_glGetFloatv = (void(*)(GLenum, GLfloat*)) glSystemOperationsTable[GLPOINTSIZE];

    // Overload the set of OpenGL functions seen by the application with.
    // our custom glColor3f call instead
    glOperationsTable[GLBEGIN] = (glOp)o_glBegin;
    glOperationsTable[GLEND] = (glOp)o_glEnd;
}

static void
o_glBegin(GLenum type)
{
    if(type == GL_POINTS){
        // Backup and change the pointsize
        s_glGetFloatv(GL_POINT_SIZE, &oldPointSize);
        s_glPointSize(pointSize);
    }
    
    // Call the 'true' glBegin function
    s_glBegin(type);
}

static void
o_glEnd(void)
{
    if(oldPointSize != 0.0){
        // restore altered pointSize
        s_glPointSize(oldPointSize);
        oldPointSize = 0.0;
    }
    
    // Call the 'true' glEnd function
    s_glEnd();
}
