/* IBM_PROLOG_BEGIN_TAG                                                   */
/* This is an automatically generated prolog.                             */
/*                                                                        */
/*                                                                        */
/*                                                                        */
/* IBM CONFIDENTIAL                                                       */
/*                                                                        */
/* OBJECT CODE ONLY SOURCE MATERIALS                                      */
/*                                                                        */
/* COPYRIGHT International Business Machines Corp. 2005, 2007             */
/* All Rights Reserved                                                    */
/*                                                                        */
/* The source code for this program is not published or otherwise         */
/* divested of its trade secrets, irrespective of what has been           */
/* deposited with the U.S. Copyright Office.                              */
/*                                                                        */
/* IBM_PROLOG_END_TAG                                                     */

// ABOUT:
// This overload provides a means to alter the runtime behaviour of 
// the fragment programs used by id Software's Doom3. 
// This allows the correct rendering of postProcess effects like 
// heat haze and glass on a scaled video wall

// DISCLAIMER:
// This software is provided without warranty, as is, for demonstration purposes only.
// Doom3 is a trademark of id Software Inc. 

#include <stdio.h>
// wire.h contains the SVN OpenGL enumerants (e.g. GLCOLOR3F etc.) and must be included
#include "wire.h"

// Overloading several functions
static void (*s_glCopyTexSubImage2D)(GLenum, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei);
static void (*s_glCopyTexSubImage2DR)(GLenum, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei);
static void  o_glCopyTexSubImage2DClient(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint x, GLint y, GLsizei width, GLsizei height);
static void  o_glCopyTexSubImage2DServer(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint x, GLint y, GLsizei width, GLsizei height);

static void (*s_glProgramEnvParameter4fvARB)(GLenum, GLuint, const GLfloat *);
static void  o_glProgramEnvParameter4fvARB(GLenum target, GLuint pname, const GLfloat *values) ;

static void (*s_glTexImage2D)(GLenum, GLint, GLint, GLsizei, GLsizei, GLint, GLenum, GLenum, const GLvoid *);

static int clientWidth  = 0;
static int clientHeight = 0;
static int serverWidth  = 0;
static int serverHeight = 0;


void Overload(glOp *glOperationsTable, glOp *glSystemOperationsTable)
{
    // Acquire a function pointer to access the DVGetDisplayName() SVN function
    char*(*_getDisplayName)() = 
        (char*(*)()) glOperationsTable[DVGETDISPLAYNAME];

    char* myDisplayName = _getDisplayName();
    int amClient = 0;


    // For Server Nodes myDisplayName will be the system's hostname as specified 
    // in the wall configuration file.
    // For the Client Application Node, myDisplayName will be "client".
    // If an error occurred myDisplayName will be NULL.
    if (myDisplayName==NULL)
    {
        fprintf(stderr,"Failed to obtain SVN system info;\n");
        return;
    }
    else if ( !strcmp(myDisplayName,"client") )
    {
        amClient = 1;
    }
    FILE *fp = fopen("d3_shaders.cfg","r");
    if(!fp){
        fprintf(stderr, "Failed to load d3_shaders.cfg file\n");
        clientWidth = 640;
        clientHeight = 480;
        serverWidth = 768;
        serverHeight = (int)(serverWidth * ((float)clientHeight / (float)clientWidth));
        //return;
    } else {
        fscanf(fp, "client %d %d\n", &clientWidth, &clientHeight);
        fscanf(fp, "server %d %*d\n", &serverWidth);
        serverHeight = (int)(serverWidth * ((float)clientHeight / (float)clientWidth));
    }

    // Acquire a pointer to the 'true' OpenGL function for future use.
    // glOperationsTable are 'true' functions with SVN internal alterations
    // glSystemOperationsTable are the functions direct from the OpenGL driver
        
    if(amClient == 1)
    {
        fprintf(stderr, "SVN client using \"D3 Shaders\" Overload for Doom3(tm)... \n");
        s_glCopyTexSubImage2DR = (void(*)(GLenum, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei)) glSystemOperationsTable [GLCOPYTEXSUBIMAGE2D];
        // Overload
        glOperationsTable[GLCOPYTEXSUBIMAGE2D] = (glOp)o_glCopyTexSubImage2DClient;
    } else {
        fprintf(stderr, "SVN server rendering on display %s using \"D3 Shaders\" Overload for Doom3(tm)... \n", myDisplayName);
        s_glTexImage2D = (void(*)(GLenum, GLint, GLint, GLsizei, GLsizei, GLint, GLenum, GLenum, const GLvoid *)) glOperationsTable [GLTEXIMAGE2D];
        s_glCopyTexSubImage2D = (void(*)(GLenum, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei)) glOperationsTable [GLCOPYTEXSUBIMAGE2D];
        s_glProgramEnvParameter4fvARB = (void(*)(GLenum, GLuint, const GLfloat *)) glOperationsTable[GLPROGRAMENVPARAMETER4FVARB];
        //glSystemOperationsTable
        // Overload
        glOperationsTable[GLCOPYTEXSUBIMAGE2D] = (glOp)o_glCopyTexSubImage2DServer;
        glOperationsTable[GLPROGRAMENVPARAMETER4FVARB] = (glOp)o_glProgramEnvParameter4fvARB;
    }
}

// On the client, just call the system function
static void
o_glCopyTexSubImage2DClient(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint x, GLint y, GLsizei width, GLsizei height)
{
    s_glCopyTexSubImage2DR(target, level, xoffset, yoffset, x, y, width, height);
}

// On the server we increase the size of Texture image the game screengrabs to
static void
o_glCopyTexSubImage2DServer(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint x, GLint y, GLsizei width, GLsizei height)
{
    int lwidth = serverWidth;
    int lheight= serverHeight;

    // For fragment programs used by Doom3 for Heat Haze and Glass Effects
    if(width == clientWidth && height == clientHeight)
    {   
        // We resize the texture to be a non-power of two texture that matches the screen dimensions
        s_glTexImage2D(GL_TEXTURE_2D, level, GL_RGBA8, lwidth, lheight, 0, GL_RGBA, GL_UNSIGNED_BYTE, NULL);
        // Copy the local screen to the texture
        s_glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 0, 0, lwidth, lheight);
        return;
    }
    else if((height == 1 && width == clientWidth) || (width == 1 && height == clientHeight) )
    {
        //else if((height == 1 && width == currDrawableInfo->avp.w) || (width == 1 && height == currDrawableInfo->avp.h) )
        if(width == 1){
            s_glCopyTexSubImage2D(target, level, lwidth-1, 0, lwidth - 1, 0, 1, lheight);
        } else {
            s_glCopyTexSubImage2D(target, level, 0, lheight-1, 0, lheight - 1, lwidth, 1);
        }
        return;
    }

    // Handle normal case - should not get here normally
    s_glCopyTexSubImage2D(target, level, xoffset, yoffset, x, y, width, height);
}

// On the servers We modify the parameters fed to the fragment program
// Two parameters need modifying
static void 
o_glProgramEnvParameter4fvARB(GLenum target, GLuint pname, const GLfloat *values) 
{
    if(target == GL_FRAGMENT_PROGRAM_ARB){
        float localValues[4];
        // Heathaze Fragment Program takes two ENV variables
        // Env0 is scaling of the captured texture to the screen
        // eg 640x480 fits into 1024x512 texture
        // We replace the texture with a 1:1 mapping, so we alter the values
        // Support 640x480, 800x600, 1024x768 clients (by default)
        if(pname == 0 && ((clientWidth == 640 && values[0] == 0.625) || (clientWidth == 800 && values[0] == 0.78125)))
        {
            localValues[0] = 1.0;
            localValues[1] = 1.0;
            localValues[2] = values[2];
            localValues[3] = values[3];
            s_glProgramEnvParameter4fvARB(target, pname, localValues);
            return;
        }
        // Env1 is the inverse of the viewport dimensions
        // e.g. 1.0/640.0, 1.0/480.0 - must be changed to match the actual dimesions
        if(pname == 1)
        {
            
            float invWidth = 1.0 / (float)clientWidth;//currDrawableInfo->avp.w;
            float epsilon = 0.00001;
            
            // Got to use this instead of a straight comparison values[0] == invWidth
            if(values[0] >= (invWidth - epsilon) && values[0] <= (invWidth + epsilon)){
                localValues[0] = 1.0 / (float)serverWidth;
                localValues[1] = 1.0 / (float)serverHeight;
            } else {
                localValues[0] = values[0];
                localValues[1] = values[1];
            }        
            localValues[2] = values[2];
            localValues[3] = values[3];
            s_glProgramEnvParameter4fvARB(target, pname, localValues);
            return;
        }
    }
    s_glProgramEnvParameter4fvARB(target, pname, values);
}
