/* IBM_PROLOG_BEGIN_TAG                                                   */
/* This is an automatically generated prolog.                             */
/*                                                                        */
/*                                                                        */
/*                                                                        */
/* Licensed Materials - Property of IBM                                   */
/*                                                                        */
/* COPYRIGHT International Business Machines Corp. 2005               */
/* All Rights Reserved                                                    */
/*                                                                        */
/* US Government Users Restricted Rights - Use, duplication or            */
/* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.      */
/*                                                                        */
/* IBM_PROLOG_END_TAG                                                     */
/* Broadcast bandwidth using the MPI_BCast function */
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <mpi.h>
#define NSIZE 8192
#define NUMIT 20

main(int argc, char **argv)
{
  int nsize = NSIZE;
  int numit = NUMIT;
  int nalloc;
  int numtask, taskid;
  int i, k, len, nbytes;
  double t1, t2;
  double t12, band, band1;
  double *x;
  char hname[256];

  gethostname(hname, sizeof(hname));
  MPI_Init(&argc,&argv);
  MPI_Comm_size(MPI_COMM_WORLD,&numtask);
  MPI_Comm_rank(MPI_COMM_WORLD,&taskid);

  printf("SVN: bcast_bandwith: Task %d running on node %s\n", taskid, hname);
  fflush(stdout);

if (taskid == 0) {
	if ((argc > 2) && (argv[2][0] != '-')) numit = atoi(argv[2]);
	if ((argc > 1) && (argv[1][0] != '-')) nsize = atoi(argv[1]);
	printf("SVN: broadcasting %d doublewords, %d iterations\n", nsize, numit);
	}

	MPI_Bcast(&numit,1,MPI_INT,0,MPI_COMM_WORLD);
	MPI_Bcast(&nsize,1,MPI_INT,0,MPI_COMM_WORLD);
	
    nalloc = nsize*sizeof(double);
    if (nalloc < 8 ) nalloc = 8;	

    x = (double*)malloc(nalloc);
	if (!x) { fprintf(stderr,"SVN: can't allocate broadcast array of size %d\n", nalloc); exit(1); }


/*  Initialize message passing parameters. */

      len   =  sizeof(double) * nsize ;

/*  Initialize x, y arrays on all nodes */

	for (i=0;i<nsize;i++) {
            x[i] = (double) ( i * (taskid + 1)); 
		}

/* Synchronize before calling timer rtc. */

    MPI_Barrier(MPI_COMM_WORLD);
	
    t1 = MPI_Wtime();

/* Do the message broadcast numit times.*/

    for (i=0;i<numit;i++) {

	  MPI_Bcast(x,len,MPI_CHAR,0,MPI_COMM_WORLD);

	}

    MPI_Barrier(MPI_COMM_WORLD);

    t2 = MPI_Wtime();

	if (taskid == 0) {
       printf("Start timer value = %6.1lf us\n",t1*1000000.);
       printf("End   timer value = %6.1lf us\n",t2*1000000.);
       t12 = (t2- t1)/(float) numit;
       band = (numtask-1)*len / t12 ;
       band1 = numtask*len / t12 ;
       printf("Average Elapsed time = %6.1lf useconds per loop\n",t12*1000000.);
       printf("BROADCAST BANDWIDTH TO OTHERS = %6.3f*10**6 Bytes/sec\n",band*.000001);
       printf("BROADCAST BANDWIDTH WITH SELF = %6.3f*10**6 Bytes/sec\n",band1*.000001);
	 }
      MPI_Finalize();
      exit(0);
}
